/******************************************************************************/
/* Files to Include                                                           */
/******************************************************************************/

#if defined(__XC)
    #include <xc.h>         /* XC8 General Include File */
#elif defined(HI_TECH_C)
    #include <htc.h>        /* HiTech General Include File */
#endif

#include <stdint.h>         /* For uint8_t definition */
#include <stdbool.h>        /* For true/false definition */

#include "user.h"

/******************************************************************************/
/* User Functions                                                             */
/******************************************************************************/

/* <Initialize variables in user.h and insert code for user algorithms.> */
void InitInterrupt(void)
{
	PIR1bits.SSP1IF = 0;
	PIE1bits.SSP1IE = 1;	
	INTCONbits.PEIE = 1;
	INTCONbits.GIE = 1;
}
void InitAddress(void)
{
	unsigned int addr;
	addr = ReadFlash(MEMORY_ADDRESS);
	if((addr & 0xFF) == 0xFF)
		ADDRESS = DEFAULT_ADDRESS;
	else
		ADDRESS = addr & 0xFF;
}
void Delay(unsigned int delay)
{
	while(delay--)
	{
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
	}
}

void InitGPIO(void)
{
	ANSELA = 0;
	ANSELC = 0;

	/* PROG PIN */
	ADDR_TRIS = 0;
	ADDR_LAT = 0;

	/* IR RECIEVE */
	TRISAbits.TRISA2 = 1;
	LATAbits.LATA2 = 0;

	/* UNUSED PINS */
	TRISCbits.TRISC3 = 0;
	LATCbits.LATC3 = 0;
	
	TRISCbits.TRISC2 = 0;
	LATCbits.LATC2 = 0;
	
	TRISAbits.TRISA4 = 0;
	LATAbits.LATA4 = 0;
	
	TRISAbits.TRISA5 = 0;
	LATAbits.LATA5 = 0;
	
	TRISAbits.TRISA0 = 0;
	LATAbits.LATA0 = 0;
	
	TRISAbits.TRISA1 = 0;
	LATAbits.LATA1 = 0;

}
void InitPWM(void)
{
	//DISABLE PWM OUTPUTS
	TRISCbits.TRISC5 = 1;

	//CLEAR PWMxCON
	PWM1CON = 0x00;

	//LOAD PWM PERIOD
	PR2 = 0x6F;	//36kHz

	//CLEAR DUTY-CYCLE
	PWM1DCH = 0x00;
	PWM1DCL &= 0x3F;

	//CONFIGURE AND START TIMER 2
	PIR1bits.TMR2IF = 0;
	T2CONbits.T2CKPS = 0;
	T2CONbits.TMR2ON = 1;

	//ENABLE PWM OUTPUT AND WAIT FOR OVERFLOW
	while(!PIR1bits.TMR2IF);
	PIR1bits.TMR2IF = 0;

	//ENABLE PWM OUTPUTS
	TRISCbits.TRISC5 = 0;
	PWM1CON |= 0x40;

	PWM1DCL = 127 << 6;
	PWM1DCH = 127 >> 2;
}
void InitTimer0(void)
{
	OPTION_REGbits.TMR0CS = 0;	//Fosc/4
	OPTION_REGbits.PSA = 0;		//Assigned prescaller.

}

void InitI2C(void)
{
	SSPCON2 = 0x01;
	SSPCON3 = 0x40;
	SSPSTAT = 0x00;
	SSPADD = 0x48 << 1;
	SSPCON1 = 0x3E;
}
void InitApp(void)
{
	InitAddress();
	InitGPIO();
	InitTimer0();
	InitI2C();
	InitPWM();
	InitInterrupt();

}
unsigned int ReadFlash(unsigned int address)
{
	int data = 0;
	PMADRL = address & 0x00FF;
	address >>= 8;
	PMADRH = address & 0x00FF;
	PMCON1bits.CFGS = 0;
	PMCON1bits.RD = 1;
	asm("NOP");
	asm("NOP");

	data = PMDATH;
	data <<= 8;
	data |= PMDATL;
	return data;
}
void EraseFlash(unsigned int address)
{
	GIE = 0;
	PMADRL = address & 0x00FF;
	address >>= 8;
	PMADRH = address & 0x00FF;
	PMCON1bits.CFGS = 0;
	PMCON1bits.FREE = 1;
	PMCON1bits.WREN = 1;
	UnlockFlash();
	PMCON1bits.WREN = 0;
	GIE = 1;
}
void UnlockFlash(void)
{
	PMCON2 = 0x55;
	PMCON2 = 0xAA;
	PMCON1bits.WR = 1;
	asm("NOP");
	asm("NOP");
}
void WriteFlash(unsigned int data, unsigned int address)
{
	GIE = 0;
	PMCON1bits.CFGS = 0;
	PMADRL = address & 0x00FF;
	address >>= 8;
	PMADRH = address & 0x00FF;
	PMCON1bits.FREE = 0;
	PMCON1bits.LWLO = 1;
	PMCON1bits.WREN = 1;
	PMDATL = data & 0x00FF;
	data >>= 8;
	PMDATH = data & 0x00FF;
	PMCON1bits.LWLO = 0;
	UnlockFlash();
	PMCON1bits.WREN = 0;
	GIE = 1;
}
void SetAddress(unsigned char addr)
{
	if(!ADDR_PORT)
	{
		EraseFlash(MEMORY_ADDRESS);
		WriteFlash(addr, MEMORY_ADDRESS);
		ADDRESS = addr;
	}
}


